try:
    set
except NameError:
    from sets import Set as set

from dosage.util import fetchUrl, fetchUrls, fetchManyUrls
from dosage.comic import Comic, FetchComicError

class BasicComicModule(object):
    '''Base class with functions for comic modules.

    Classes should define the following attributes:
        latestUrl 
        imageUrl
        imageSearch
        prevSearch
    '''
    referrer = None
    help = 'Sorry, no help for this module yet.'

    def __init__(self):
        self.name = self.__class__.__name__.split('.')[-1]
        self.currentUrl = None
        self.urls = set()

    def getFilename(self, imageUrl, pageUrl):
        return None

    def getReferrer(self, imageUrl, pageUrl):
        return self.referrer or self.latestUrl
        
    def getComic(self, url, pageUrl):
        if not url:
            return None
        return Comic(self.name, url, filename=self.getFilename(url, pageUrl), referrer=self.getReferrer(url, pageUrl))

    def getCurrentComics(self):
        self.currentUrl = self.latestUrl
        return self.getNextComics()

    def getNextComics(self):
        comics = []
        while not comics and self.currentUrl and self.currentUrl not in self.urls:
            comicUrlGroups, prevUrl = fetchManyUrls(self.currentUrl, (self.imageSearch, self.prevSearch))
            prevUrl = prevUrl and prevUrl[0] or None
            for comicUrl in comicUrlGroups:
                comics.append(self.getComic(comicUrl, self.currentUrl))
            self.urls.update([self.currentUrl])
            self.currentUrl = (prevUrl, None)[prevUrl in self.urls]
        return comics
    
    def setStrip(self, index):
        self.currentUrl = self.imageUrl % index

    def getHelp(self):
        return self.help

    def __iter__(self):
        """Iterate through the strips, starting from the current one and going backward."""
        if not self.currentUrl:
            self.currentUrl = self.latestUrl

        comics = True
        while comics:
            comics = self.getNextComics()
            if comics:
                yield comics

class BounceMixin(object):
    '''
    Mixin for comics modules needing the "bounce" trick; basically simulating
    going backwards, and then forwards again to get the canonical page URL.

    Classes using this mixin should define:

        baseUrl: the URL to apply prevSearch on
        prevSearch: a compiled regular expression for finding the 'previous' URL.
        nextSearch: a compiled regular expression for finding the 'next' URL.
    '''

    __latestUrl = None

    def getLatestUrl(self):
	if not self.__latestUrl:
	    self.__latestUrl = fetchUrl(self.baseUrl, self.prevSearch)
	    self.__latestUrl = fetchUrl(self.__latestUrl, self.nextSearch)
	return self.__latestUrl

    latestUrl = property(getLatestUrl)

class IndirectLatestMixin(object):
    '''
    Mixin for comics modules that link to the latest comic from a base page of
    some kind.

    Classes using this mixin should provide:

        baseUrl: the URL where the link to the latest comic is found
        latestSearch: a compiled regex for finding the 'latest' URL.

    '''

    __latestUrl = None
    
    def getLatestUrl(self):
        if not self.__latestUrl:
            self.__latestUrl = fetchUrl(self.baseUrl, self.latestSearch)
        return self.__latestUrl

    latestUrl = property(getLatestUrl)
